global organism
global idr_base_url
global hostname
global username
global taxonomy_id
global ensemble_id
global gene_symbol
organism = 'Homo Sapiens'
taxonomy_id = '9606'
ensemble_id = 'ENSG00000106526'
gene_symbol = 'ACTR3C'
go_gene_list = ['SOD1', 'SOD2', 'SO']
